local ADDON, Package = ...
local Color = {}

function Color.to_hsl_from_rgb(r, g, b)
  local min, max = math.min(r, g, b), math.max(r, g, b)
  local h, s, l = 0, 0, (max + min) / 2

  if min == max then
    h, s = 0, 0
  else
    local delta = max - min

    s = l > 0.5 and delta / (2 - max - min) or delta / (max + min)

    if max == r then
      h = (g - b) / delta + (g < b and 6 or 0)
    elseif max == g then
      h = (b - r) / delta + 2
    elseif max == b then
      h = (r - g) / delta + 4
    end

    h = h / 6
  end

  return h, s, l
end

function Color.to_rgb_from_hue(p, q, t)
  if t < 0 then
    t = t + 1
  end

  if t > 1 then
    t = t - 1
  end

  if t < 1 / 6 then
    return p + (q - p) * 6 * t
  end

  if t < 1 / 2 then
    return q
  end

  if t < 2 / 3 then
    return p + (q - p) * (2 / 3 - t) * 6
  end

  return p
end

function Color.to_rgb_from_hsl(h, s, l)
  local r, g, b = 0, 0, 0

  if s == 0 then
    r, g, b = l, l, l
  else
    local q = l < 0.5 and l * (1 + s) or l + s - l * s
    local p = 2 * l - q

    r = Color.to_rgb_from_hue(p, q, h + 1 / 3)
    g = Color.to_rgb_from_hue(p, q, h)
    b = Color.to_rgb_from_hue(p, q, h - 1 / 3)
  end

  return r, g, b
end

function Color.to_rgb_from_hex(hex)
  local r = bit.band(bit.rshift(hex, 16), 0xFF)
  local g = bit.band(bit.rshift(hex, 8), 0xFF)
  local b = bit.band(hex, 0xFF)

  return r / 255.0, g / 255.0, b / 255.0
end

function Color.to_rgba_from_hex(hex)
  local a = bit.band(hex, 0xFF) / 255.0
  local r, g, b = Color.to_rgb_from_hex(bit.rshift(hex, 8))

  return r, g, b, a
end

function Color.to_hex_string_from_rgb(r, g, b)
  return string.format("%.2x%.2x%.2x", r * 255, g * 255, b * 255)
end

function Color.to_hex_markup_from_rgb(r, g, b)
  return "|cff" .. Color.to_hex_string_from_rgb(r, g, b)
end

function Color.format(any, text)
  if type(any) == "table" then
    return string.format("|cff%s%s|r", Color.to_hex_string_from_rgb(any.r, any.g, any.b), text)
  end

  if type(any) == "number" then
    return string.format("|cff%6x%s|r", any, text)
  end

  if type(any) == "string" then
    return string.format("|cff%s%s|r", any, text)
  end
end

function Color.darken(r, g, b, amount)
  local h, s, l = Color.to_hsl_from_rgb(r, g, b)

  l = l - (amount / 100)
  l = math.min(1, math.max(0, l))

  return Color.to_rgb_from_hsl(h, s, l)
end

Package.Color = Color
