local ADDON, Package = ...
local Color = Package.Color
local ColorPalette = Package.ColorPalette

local function gametooltip_item(is_item_icon_flagged, is_item_stack_count_flagged, is_item_id_flagged, is_item_level_flagged, is_item_price_flagged)
  local TEXT_LEFTS = {
    GameTooltip = GameTooltipTextLeft1,
    ItemRefTooltip = ItemRefTooltipTextLeft1,
    ShoppingTooltip1 = ShoppingTooltip1TextLeft2,
    ShoppingTooltip2 = ShoppingTooltip2TextLeft2,
  }

  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
    ShoppingTooltip1 = ShoppingTooltip1TextRight2,
    ShoppingTooltip2 = ShoppingTooltip2TextRight2,
  }

  local left_id = nil
  local left_stash = ""

  local right_id = nil
  local right_stash = ""

  local function OnTooltipSetItem(self)
    local _, item_link = self:GetItem()

    if not item_link then
      return nil
    end

    local item_id = GetItemInfoFromHyperlink(item_link)
    local item_name, _, _, item_level, _, _, _, item_stack_count, _, item_icon, item_sell_price = GetItemInfo(item_id)

    if not item_name then
      return nil
    end

    local left = TEXT_LEFTS[self:GetName()]
    local left_text = left:GetText() or ""

    local right = TEXT_RIGHTS[self:GetName()]
    local right_text = right:GetText() or ""

    if left_id ~= item_id then
      if is_item_icon_flagged then
        local item_icon_markup = CreateTextureMarkup(item_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

        left_text = item_icon_markup .. " " ..left_text
      end

      if is_item_stack_count_flagged and item_stack_count > 1 then
        left_text = left_text .. " (" .. item_stack_count .. ")"
      end

      left_id = item_id
      left_stash = left_text
    end

    left:SetText(left_stash)

    if right_id ~= item_id then
      if is_item_id_flagged then
        right_text = right_text .. Color.format(ColorPalette.GRAY, item_id)
      end

      if is_item_level_flagged then
        right_text = right_text .. Color.format(ColorPalette.GOLD, " " .. LEVEL .. " " .. item_level)
      end

      right_id = item_id
      right_stash = right_text
    end

    right:SetText(right_stash)
    right:Show()

    if is_item_price_flagged then
      local container = GetMouseFocus()
      local item_count = 1

      if type(container) == "table" and type(container.count) == "number" then
        item_count = container.count
      end

      if item_count and item_count > 0 and item_sell_price and item_sell_price > 0 and self.shownMoneyFrames == nil then
        SetTooltipMoney(self, item_count * item_sell_price, "STATIC", SELL_PRICE .. ":")
      end
    end
  end

  if is_item_icon_flagged or is_item_stack_count_flagged or is_item_id_flagged or is_item_level_flagged or is_item_price_flagged then
    GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ItemRefTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ShoppingTooltip1:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ShoppingTooltip2:HookScript("OnTooltipSetItem", OnTooltipSetItem)
  end
end

local function gametooltip_spell(is_spell_icon_flagged, is_spell_id_flagged)
  local TEXT_LEFTS = {
    GameTooltip = GameTooltipTextLeft1,
    ItemRefTooltip = ItemRefTooltipTextLeft1,
  }

  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
  }

  local left_id = nil
  local left_stash = ""

  local right_id = nil
  local right_stash = ""

  local function OnTooltipSetSpell(self)
    local _, spell_id = self:GetSpell()

    if not spell_id then
      return nil
    end

    local left = TEXT_LEFTS[self:GetName()]
    local left_text = left:GetText() or ""

    local right = TEXT_RIGHTS[self:GetName()]
    local right_text = right:GetText() or ""

    if is_spell_icon_flagged and not left_id ~= spell_id then
      local spell_icon = GetSpellTexture(spell_id)
      local spell_icon_markup = CreateTextureMarkup(spell_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

      left_id = spell_id
      left_stash = spell_icon_markup .. " " .. left_text
    end

    left:SetText(left_stash)

    if is_spell_id_flagged and not right_id ~= spell_id then
      local spell_id_text = Color.format(ColorPalette.GRAY, spell_id)

      right_id = spell_id
      right_stash = spell_id_text .. " " .. right_text
    end

    right:SetText(right_stash)
    right:Show()
  end

  if is_spell_icon_flagged or is_spell_id_flagged then
    GameTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetSpell)
    ItemRefTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetSpell)
  end
end

local function gametooltip_aura(is_aura_id_flagged, is_aura_icon_flagged, is_aura_caster_flagged)
  local left = GameTooltipTextLeft1
  local right = GameTooltipTextRight1

  local left_id = nil
  local left_stash = ""

  local right_id = nil
  local right_stash = ""

  local function process(self)
    local _, aura_icon, _, _, _, _, aura_caster_id, _, _, aura_id = UnitAura(self.unit, self:GetID(), self.filter)

    if left_id ~= aura_id then
      local left_text = left:GetText() or ""

      if is_aura_icon_flagged and aura_icon then
        local aura_icon_markup = CreateTextureMarkup(aura_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

        left_text = aura_icon_markup .. " " .. left_text
      end

      if is_aura_caster_flagged and aura_caster_id then
        local aura_caster_name = UnitName(aura_caster_id)
        local aura_caster_class = UnitClassBase(aura_caster_id)

        if aura_caster_name and aura_caster_class then
          left_text = left_text .. RAID_CLASS_COLORS[aura_caster_class]:WrapTextInColorCode(" <" .. aura_caster_name .. ">")
        end
      end

      left_id = aura_id
      left_stash = left_text
    end

    if right_id ~= aura_id then
      local right_text = right:GetText() or ""

      if is_aura_id_flagged and aura_id then
        right_text = Color.format(ColorPalette.GRAY, aura_id) .. " " .. right_text
      end

      right_id = aura_id
      right_stash = right_text
    end

    left:SetText(left_stash)

    right:SetText(right_stash)
    right:Show()

    GameTooltip:Show()
  end

  if is_aura_id_flagged or is_aura_icon_flagged or is_aura_caster_flagged then
    local function OnUpdate(self, elapsed)
      if BuffFrame.BuffFrameUpdateTime <= 0 then
        local owner = GameTooltip:GetOwner()

        if owner and owner:GetParent() == BuffFrame and GameTooltip:IsShown() then
          process(owner)
        end
      end
    end

    local agent = CreateFrame("Frame")
    local auras = {}

    local function AuraButton_Update(button_name, index, filter)
      local button_frame_name = button_name .. index
      local button = _G[button_frame_name]

      if button and not auras[button_frame_name] then
        local OnEnter = button:GetScript("OnEnter")
        local OnLeave = button:GetScript("OnLeave")

        local function OnEnter_Proxy(self)
          OnEnter(self)
          process(self)
          agent:SetScript("OnUpdate", OnUpdate)
        end

        local function OnLeave_Proxy(self)
          OnLeave(self)
          agent:SetScript("OnUpdate", nil)
        end

        button:SetScript("OnEnter", OnEnter_Proxy)
        button:SetScript("OnLeave", OnLeave_Proxy)

        auras[button_frame_name] = true
      end
    end

    hooksecurefunc("AuraButton_Update", AuraButton_Update)
  end
end

local function gametooltip_unit(is_unit_class_color_flagged, is_unit_guild_color_flagged)
  local function OnTooltipSetUnit(self)
    local _, unit_id = self:GetUnit()

    if not unit_id or not UnitIsPlayer(unit_id) then
      return nil
    end

    if is_unit_class_color_flagged then
      local left = _G[self:GetName() .. "TextLeft1"]

      if left then
        local left_text = left:GetText() or ""

        local unit_class = UnitClassBase(unit_id)
        local unit_class_color = RAID_CLASS_COLORS[unit_class]

        if unit_class and unit_class_color then
          left:SetFormattedText("|c%s%s|r", unit_class_color.colorStr, left_text)
        end
      end
    end

    if is_unit_guild_color_flagged then
      local left = _G[self:GetName() .. "TextLeft2"]

      if left then
        local left_text = left:GetText() or ""

        local unit_guild_name = GetGuildInfo(unit_id)

        if unit_guild_name then
          left:SetFormattedText("|cff009819%s|r", unit_guild_name)
        end
      end
    end
  end

  if is_unit_class_color_flagged or is_unit_guild_color_flagged then
    GameTooltip:HookScript("OnTooltipSetUnit", OnTooltipSetUnit)
  end
end

local function gametooltip_anchor(position)
  local function GameTooltip_SetDefaultAnchor(tooltip, parent)
    if tooltip and parent then
      tooltip:ClearAllPoints()
      tooltip:SetOwner(parent, "ANCHOR_NONE")
      tooltip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", position[1], position[2])
    end
  end

  hooksecurefunc("GameTooltip_SetDefaultAnchor", GameTooltip_SetDefaultAnchor)
end

Package.gametooltip_item = gametooltip_item
Package.gametooltip_spell = gametooltip_spell
Package.gametooltip_aura = gametooltip_aura
Package.gametooltip_unit = gametooltip_unit
Package.gametooltip_anchor = gametooltip_anchor
